from django.db import models, router
from django.conf import settings

from isapilib.external.utilities import execute_query
from isapilib.api.models import UserAPI


class DynamicManager(models.Manager):
    _alias = None

    def using(self, alias):
        self._alias = alias
        return super().using(alias)

    def get_queryset(self):
        queryset = super().get_queryset()

        if isinstance(queryset.model._meta.db_table, dict):
            version = settings.DATABASES[self._alias]['INTELISIS_VERSION']
            db_table_options = queryset.model._meta.db_table

            try:
                queryset.model._meta.db_table = db_table_options[version]
            except KeyError:
                raise Exception(f'El modelo {queryset.model} no tiene una tabla para la versión {version}')

        return queryset


class BaseModel(models.Model):
    objects = DynamicManager()

    def _add_field(self, name):
        new_field = models.TextField(db_column=name)
        new_field.contribute_to_class(self, name)

    def get(self, name):
        if name not in [field.attname for field in self._meta.get_fields()]:
            self._add_field(name)

        return getattr(self, name)

    def set(self, name, value):
        if name not in [field.attname for field in self._meta.get_fields()]:
            self._add_field(name)

        setattr(self, name, value)

    def get_triggers(self, disabled=0, using=None):
        using = using or router.db_for_write(None)
        tb_name = self._meta.db_table
        query = 'SELECT name FROM sys.triggers WHERE parent_id = OBJECT_ID(%s) AND is_disabled = %s'
        response = execute_query(query, [tb_name, disabled], using=using)
        return response[0] if 0 < len(response) else []

    def enable_trigger(self, name, using=None):
        using = using or router.db_for_write(None)
        try:
            tb_name = self._meta.db_table
            execute_query(f'ENABLE TRIGGER [{name}] ON [{tb_name}]', using=using)
        except Exception:
            pass

    def disable_trigger(self, name, using=None):
        using = using or router.db_for_write(None)
        try:
            tb_name = self._meta.db_table
            execute_query(f'DISABLE TRIGGER [{name}] ON [{tb_name}]', using=using)
        except Exception:
            pass

    def save(self, *args, **kwargs):
        check_triggers = kwargs.pop('triggers', False)
        using = kwargs.get('using', router.db_for_write(None))
        triggers = None
        if not check_triggers:
            triggers = self.get_triggers(using=using)
            for tgr in triggers:
                self.disable_trigger(tgr[0], using=using)

        try:
            return super().save(*args, **kwargs)
        except Exception as e:
            raise e
        finally:
            if not check_triggers:
                for tgr in triggers:
                    self.enable_trigger(tgr[0], using=using)

    class Meta:
        abstract = True



import time

from django.conf import settings
from django.http import JsonResponse
from rest_framework.request import Request

from isapilib.core.decorators import safe_method
from isapilib.logger.utilities import insert_log


def logger_method(tipo, interfaz=None, log_all=False):
    interfaz = interfaz or getattr(settings, 'INTERFAZ_NAME', '')

    def decorador(func):
        def wrapper(*args, **kwargs):
            start = time.time()
            request = next((arg for arg in args if isinstance(arg, Request)), None)

            try:
                response: JsonResponse = func(*args, **kwargs)
                end = time.time()
                if (hasattr(response, 'status_code') and response.status_code not in range(200, 300)) or log_all:
                    response_time = (end - start) * 1000
                    insert_log(request=request, response=response, interfaz=interfaz, tipo=tipo, time=response_time)
                return response
            except Exception as e:
                insert_log(request=request, response=str(e), interfaz=interfaz, tipo=tipo)
                raise e

        return wrapper

    return decorador


def logger(tipo, interfaz=None, log_all=False):
    interfaz = interfaz or getattr(settings, 'INTERFAZ_NAME', '')

    def class_decorator(cls):
        method_names = ['get', 'post', 'put', 'delete']
        for method_name in method_names:
            if not hasattr(cls, method_name): continue
            method = getattr(cls, method_name)
            log_tipo = f'{tipo} {method_name.upper()}'
            setattr(cls, method_name, logger_method(log_tipo, interfaz, log_all)(safe_method(method)))
        return cls

    return class_decorator



from datetime import datetime
from typing import List

from django.db import router, connections


class ExecutionQuery:
    def __init__(self, name, params, using):
        self.name = name
        self.params = params or []
        self.using = using or router.db_for_write(None)

    @property
    def cursor(self):
        if not hasattr(self, '_cursor'):
            self._cursor = connections[self.using].cursor()
        return self._cursor

    def get_query(self):
        return self.name

    def get_params(self):
        return self.params

    def _execute(self, query, params, using):
        self.cursor.execute(query, params)
        results = []
        try:
            while True:
                rows = self.cursor.fetchall()
                if rows:
                    results.append(rows)
                if not self.cursor.nextset():
                    break
        except Exception as e:
            pass

        return results

    def execute(self):
        return self._execute(self.get_query(), self.get_params(), self.using)


class ExecutionFN(ExecutionQuery):
    def get_query(self):
        fn_params = self.get_params()
        return f'''SELECT dbo.{self.name}({', '.join(['%s' for _ in fn_params])})'''

    def execute(self):
        results = super().execute()
        while isinstance(results, (list, tuple)):
            results = results[0]
        return results


class ExecutionSP(ExecutionQuery):
    def __init__(self, name, params, with_transaction=False, using=None, ):
        super().__init__(name, params, using)
        self.with_transaction = with_transaction

    @property
    def output_variables(self) -> List[str]:
        if not hasattr(self, '_output_variables'):
            query = "SELECT PARAMETER_NAME, DATA_TYPE, CHARACTER_MAXIMUM_LENGTH FROM information_schema.parameters WHERE SPECIFIC_NAME = %s AND PARAMETER_MODE = 'INOUT'"
            response = self._execute(query, [self.name])
            self._output_variables = response[0] if 0 < len(response) else []
        return self._output_variables or []

    def get_execution_settings(self):
        execution_settings = ['SET NOCOUNT ON;']
        if self.with_transaction: execution_settings.append('BEGIN TRANSACTION;')
        return '\n'.join(execution_settings)

    def declare_variable(self, name, type, length):
        declaration = f'{name} '

        if type in ('varchar', 'char'):
            declaration += f"{type}({'MAX' if length == -1 else length})"
        else:
            declaration += f'{type}'

        return declaration

    def get_declaration_output_variables(self):
        if len(self.output_variables) == 0: return ''
        variables = [self.declare_variable(name, type, length) for name, type, length in self.output_variables]
        return f"DECLARE {', '.join(variables)};"

    def get_sp_parameters(self):
        if isinstance(self.params, dict):
            parameters = ', '.join([f'@{i}=%s' for i in self.params.keys()])
        else:
            parameters = ', '.join(['%s' for _ in self.params])

        output_variables = ', '.join([f'{data[0]}={data[0]} OUTPUT' for data in self.output_variables])
        if output_variables != '': output_variables = ', ' + output_variables
        return f'{parameters}{output_variables}'

    def get_sp_exec(self):
        parameters = self.get_sp_parameters()
        if parameters == '': return f'EXEC {self.name};'
        return f'EXEC {self.name} {parameters};'

    def get_select_for_results(self):
        if len(self.output_variables) == 0: return ''
        variables = ', '.join([data[0] for data in self.output_variables])
        return f"SELECT 'OUTPUT', {variables};"

    def get_extra(self):
        extra = []
        if self.with_transaction: extra.append('COMMIT;')
        return '\n'.join(extra)

    def get_query(self):
        execution_settings = self.get_execution_settings()
        declaration_output_variables = self.get_declaration_output_variables()
        exec_sp = self.get_sp_exec()
        results = self.get_select_for_results()
        extra = self.get_extra()
        return '\n'.join([
            execution_settings,
            declaration_output_variables,
            exec_sp,
            results,
            extra
        ]).strip()

    def to_internal_value(self, value):
        if isinstance(value, datetime):
            return value.strftime('%Y-%m-%dT%H:%M:%S')
        return value

    def to_internal_values(self, values):
        return [self.to_internal_value(value) for value in values]

    def get_params(self):
        values = super().get_params()
        if isinstance(values, dict):
            values = values.values()
        return self.to_internal_values(values)

    def execute(self, only_output=True):
        results = super().execute()
        if only_output:
            last_line = results[-1][0]
            if last_line[0] == 'OUTPUT':
                return last_line[1:]
        return results


def execute_query(query, params=None, using=None):
    return ExecutionQuery(query, params, using).execute()


def execute_fn(fn_name, fn_params=None, using=None):
    return ExecutionFN(fn_name, fn_params, using).execute()


def execute_sp(sp_name, sp_params=None, only_output=True, with_transaction=False, using=None):
    return ExecutionSP(sp_name, sp_params, with_transaction, using).execute(only_output)


def get_sucursal(modulo='VTAS', mov='Servicio', sucursal=0, using=None):
    from isapilib.models import Sucursal
    if issubclass(type(sucursal), Sucursal): sucursal = sucursal.pk
    pk = execute_fn("fnCA_GeneraSucursalValida", [modulo, mov, sucursal], using=using)
    return Sucursal.objects.get(pk=pk)


def get_almacen(modulo='VTAS', mov='Servicio', sucursal=0, using=None):
    from isapilib.models import Sucursal, Almacen
    if issubclass(type(sucursal), Sucursal): sucursal = sucursal.pk
    pk = execute_fn("fnCA_GeneraAlmacenlValido", [modulo, mov, sucursal], using=using)
    return Almacen.objects.get(pk=pk)


def get_uen(modulo='VTAS', mov='Servicio', sucursal=0, concepto='Publico', using=None):
    from isapilib.models import Sucursal
    if issubclass(type(sucursal), Sucursal): sucursal = sucursal.pk
    return execute_fn("fnCA_GeneraUENValida", [modulo, mov, sucursal, concepto], using=using)


def get_param_empresa(interfaz, clave, default=None, using=None):
    result = execute_fn('fnCA_BusquedaClaveParametroInterfazEmpresa', [interfaz, clave], using=using)
    return result if result is not None and result != '' else default


def get_param_sucursal(sucursal, clave, default=None, using=None):
    from isapilib.models import Sucursal
    if issubclass(type(sucursal), Sucursal): sucursal = sucursal.pk
    result = execute_fn('fnCA_CatParametrosSucursalValor', [sucursal, clave], using=using)
    return result if result is not None and result != '' else default



# Generated by Django 5.0.1 on 2024-02-19 11:39

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='Agente',
            fields=[
                ('agente', models.CharField(db_column='Agente', max_length=10, primary_key=True, serialize=False)),
                ('nombre', models.CharField(blank=True, db_column='Nombre', max_length=100, null=True)),
                ('tipo', models.CharField(blank=True, db_column='Tipo', max_length=15, null=True)),
                ('categoria', models.CharField(blank=True, db_column='Categoria', max_length=50, null=True)),
                ('familia', models.CharField(blank=True, db_column='Familia', max_length=50, null=True)),
                ('zona', models.CharField(blank=True, db_column='Zona', max_length=30, null=True)),
                ('grupo', models.CharField(blank=True, db_column='Grupo', max_length=50, null=True)),
                ('estatus', models.CharField(db_column='Estatus', max_length=15)),
                ('alta', models.DateTimeField(blank=True, db_column='Alta', null=True)),
                ('baja', models.DateTimeField(blank=True, db_column='Baja', null=True)),
                ('direccion', models.CharField(blank=True, db_column='Direccion', max_length=100, null=True)),
                ('colonia', models.CharField(blank=True, db_column='Colonia', max_length=255, null=True)),
                ('poblacion', models.CharField(blank=True, db_column='Poblacion', max_length=30, null=True)),
                ('estado', models.CharField(blank=True, db_column='Estado', max_length=30, null=True)),
                ('pais', models.CharField(blank=True, db_column='Pais', max_length=30, null=True)),
                ('codigo_postal', models.CharField(blank=True, db_column='CodigoPostal', max_length=15, null=True)),
                ('rfc', models.CharField(blank=True, db_column='RFC', max_length=20, null=True)),
                ('curp', models.CharField(blank=True, db_column='CURP', max_length=30, null=True)),
                ('email', models.CharField(blank=True, db_column='eMail', max_length=50, null=True)),
            ],
            options={
                'db_table': 'Agente',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Art',
            fields=[
                ('articulo', models.CharField(db_column='Articulo', max_length=20, primary_key=True, serialize=False)),
                ('descripcion1', models.CharField(blank=True, db_column='Descripcion1', max_length=100, null=True)),
                ('descripcion2', models.CharField(blank=True, db_column='Descripcion2', max_length=255, null=True)),
                ('grupo', models.CharField(blank=True, db_column='Grupo', max_length=50, null=True)),
                ('categoria', models.CharField(blank=True, db_column='Categoria', max_length=50, null=True)),
                ('impuesto1', models.FloatField(db_column='Impuesto1')),
                ('impuesto2', models.FloatField(blank=True, db_column='Impuesto2', null=True)),
                ('impuesto3', models.FloatField(blank=True, db_column='Impuesto3', null=True)),
                ('tipo', models.CharField(db_column='Tipo', max_length=20)),
                ('precio_lista', models.DecimalField(blank=True, db_column='PrecioLista', decimal_places=4, max_digits=19, null=True)),
                ('estatus', models.CharField(db_column='Estatus', max_length=15)),
                ('usuario', models.CharField(blank=True, db_column='Usuario', max_length=10, null=True)),
                ('precio2', models.DecimalField(blank=True, db_column='Precio2', decimal_places=4, max_digits=19, null=True)),
                ('precio3', models.DecimalField(blank=True, db_column='Precio3', decimal_places=4, max_digits=19, null=True)),
                ('precio4', models.DecimalField(blank=True, db_column='Precio4', decimal_places=4, max_digits=19, null=True)),
                ('precio5', models.DecimalField(blank=True, db_column='Precio5', decimal_places=4, max_digits=19, null=True)),
                ('precio6', models.DecimalField(blank=True, db_column='Precio6', decimal_places=4, max_digits=19, null=True)),
                ('precio7', models.DecimalField(blank=True, db_column='Precio7', decimal_places=4, max_digits=19, null=True)),
                ('precio8', models.DecimalField(blank=True, db_column='Precio8', decimal_places=4, max_digits=19, null=True)),
                ('precio9', models.DecimalField(blank=True, db_column='Precio9', decimal_places=4, max_digits=19, null=True)),
                ('precio10', models.DecimalField(blank=True, db_column='Precio10', decimal_places=4, max_digits=19, null=True)),
                ('situacion', models.CharField(blank=True, db_column='Situacion', max_length=50, null=True)),
                ('tipo_compra', models.CharField(blank=True, db_column='TipoCompra', max_length=20, null=True)),
                ('retencion1', models.FloatField(blank=True, db_column='Retencion1', null=True)),
                ('retencion2', models.FloatField(blank=True, db_column='Retencion2', null=True)),
                ('retencion3', models.FloatField(blank=True, db_column='Retencion3', null=True)),
                ('modelo', models.CharField(blank=True, db_column='Modelo', max_length=4, null=True)),
                ('direccion', models.CharField(blank=True, db_column='Direccion', max_length=100, null=True)),
                ('direccion_numero', models.CharField(blank=True, db_column='DireccionNumero', max_length=20, null=True)),
                ('direccion_numero_int', models.CharField(blank=True, db_column='DireccionNumeroInt', max_length=20, null=True)),
                ('observaciones', models.CharField(blank=True, db_column='Observaciones', max_length=100, null=True)),
                ('colonia', models.CharField(blank=True, db_column='Colonia', max_length=100, null=True)),
                ('delegacion', models.CharField(blank=True, db_column='Delegacion', max_length=100, null=True)),
                ('poblacion', models.CharField(blank=True, db_column='Poblacion', max_length=100, null=True)),
                ('estado', models.CharField(blank=True, db_column='Estado', max_length=30, null=True)),
                ('pais', models.CharField(blank=True, db_column='Pais', max_length=30, null=True)),
                ('codigo_postal', models.CharField(blank=True, db_column='CodigoPostal', max_length=15, null=True)),
                ('tipo_impuesto1', models.CharField(blank=True, db_column='TipoImpuesto1', max_length=10, null=True)),
                ('tipo_impuesto2', models.CharField(blank=True, db_column='TipoImpuesto2', max_length=10, null=True)),
                ('tipo_impuesto3', models.CharField(blank=True, db_column='TipoImpuesto3', max_length=10, null=True)),
            ],
            options={
                'db_table': 'Art',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Cte',
            fields=[
                ('cliente', models.CharField(db_column='Cliente', max_length=10, primary_key=True, serialize=False)),
                ('nombre', models.CharField(blank=True, db_column='Nombre', max_length=254, null=True)),
                ('nombre_corto', models.CharField(blank=True, db_column='NombreCorto', max_length=20, null=True)),
                ('direccion', models.CharField(blank=True, db_column='Direccion', max_length=100, null=True)),
                ('direccion_numero', models.CharField(blank=True, db_column='DireccionNumero', max_length=50, null=True)),
                ('observaciones', models.CharField(blank=True, db_column='Observaciones', max_length=100, null=True)),
                ('delegacion', models.CharField(blank=True, db_column='Delegacion', max_length=100, null=True)),
                ('colonia', models.CharField(blank=True, db_column='Colonia', max_length=100, null=True)),
                ('poblacion', models.CharField(blank=True, db_column='Poblacion', max_length=100, null=True)),
                ('estado', models.CharField(blank=True, db_column='Estado', max_length=30, null=True)),
                ('pais', models.CharField(blank=True, db_column='Pais', max_length=100, null=True)),
                ('zona', models.CharField(blank=True, db_column='Zona', max_length=30, null=True)),
                ('codigo_postal', models.CharField(blank=True, db_column='CodigoPostal', max_length=15, null=True)),
                ('rfc', models.CharField(blank=True, db_column='RFC', max_length=15, null=True)),
                ('curp', models.CharField(blank=True, db_column='CURP', max_length=30, null=True)),
                ('telefonos', models.CharField(blank=True, db_column='Telefonos', max_length=100, null=True)),
                ('telefonos_lada', models.CharField(blank=True, db_column='TelefonosLada', max_length=6, null=True)),
                ('contacto1', models.CharField(blank=True, db_column='Contacto1', max_length=50, null=True)),
                ('contacto2', models.CharField(blank=True, db_column='Contacto2', max_length=50, null=True)),
                ('extencion1', models.CharField(blank=True, db_column='Extencion1', max_length=10, null=True)),
                ('extencion2', models.CharField(blank=True, db_column='Extencion2', max_length=10, null=True)),
                ('email1', models.CharField(blank=True, db_column='eMail1', max_length=50, null=True)),
                ('email2', models.CharField(blank=True, db_column='eMail2', max_length=50, null=True)),
                ('categoria', models.CharField(blank=True, db_column='Categoria', max_length=50, null=True)),
                ('tipo', models.CharField(blank=True, db_column='Tipo', max_length=15, null=True)),
                ('situacion', models.CharField(blank=True, db_column='Situacion', max_length=50, null=True)),
                ('agente', models.CharField(blank=True, db_column='Agente', max_length=10, null=True)),
                ('agente_servicio', models.CharField(blank=True, db_column='AgenteServicio', max_length=10, null=True)),
                ('estatus', models.CharField(db_column='Estatus', max_length=15)),
                ('ultimo_cambio', models.DateTimeField(blank=True, db_column='UltimoCambio', null=True)),
                ('descripcion1', models.CharField(blank=True, db_column='Descripcion1', max_length=50, null=True)),
                ('descripcion2', models.CharField(blank=True, db_column='Descripcion2', max_length=50, null=True)),
                ('descripcion3', models.CharField(blank=True, db_column='Descripcion3', max_length=50, null=True)),
                ('descripcion4', models.CharField(blank=True, db_column='Descripcion4', max_length=50, null=True)),
                ('descripcion5', models.CharField(blank=True, db_column='Descripcion5', max_length=50, null=True)),
                ('descripcion6', models.CharField(blank=True, db_column='Descripcion6', max_length=50, null=True)),
                ('descripcion7', models.CharField(blank=True, db_column='Descripcion7', max_length=50, null=True)),
                ('descripcion8', models.CharField(blank=True, db_column='Descripcion8', max_length=50, null=True)),
                ('descripcion9', models.CharField(blank=True, db_column='Descripcion9', max_length=50, null=True)),
                ('descripcion10', models.CharField(blank=True, db_column='Descripcion10', max_length=50, null=True)),
                ('descripcion11', models.CharField(blank=True, db_column='Descripcion11', max_length=50, null=True)),
                ('descripcion12', models.CharField(blank=True, db_column='Descripcion12', max_length=50, null=True)),
                ('descripcion13', models.CharField(blank=True, db_column='Descripcion13', max_length=50, null=True)),
                ('descripcion14', models.CharField(blank=True, db_column='Descripcion14', max_length=50, null=True)),
                ('descripcion15', models.CharField(blank=True, db_column='Descripcion15', max_length=50, null=True)),
                ('descripcion16', models.CharField(blank=True, db_column='Descripcion16', max_length=50, null=True)),
                ('descripcion17', models.CharField(blank=True, db_column='Descripcion17', max_length=50, null=True)),
                ('descripcion18', models.CharField(blank=True, db_column='Descripcion18', max_length=50, null=True)),
                ('descripcion19', models.CharField(blank=True, db_column='Descripcion19', max_length=50, null=True)),
                ('descripcion20', models.CharField(blank=True, db_column='Descripcion20', max_length=50, null=True)),
                ('personal_nombres', models.CharField(blank=True, db_column='PersonalNombres', max_length=50, null=True)),
                ('personal_apellido_paterno', models.CharField(blank=True, db_column='PersonalApellidoPaterno', max_length=50, null=True)),
                ('personal_apellido_materno', models.CharField(blank=True, db_column='PersonalApellidoMaterno', max_length=50, null=True)),
                ('personal_direccion', models.CharField(blank=True, db_column='PersonalDireccion', max_length=100, null=True)),
                ('personal_entrecalles', models.CharField(blank=True, db_column='PersonalEntreCalles', max_length=100, null=True)),
                ('personal_plano', models.CharField(blank=True, db_column='PersonalPlano', max_length=15, null=True)),
                ('personal_delegacion', models.CharField(blank=True, db_column='PersonalDelegacion', max_length=100, null=True)),
                ('personal_colonia', models.CharField(blank=True, db_column='PersonalColonia', max_length=100, null=True)),
                ('personal_poblacion', models.CharField(blank=True, db_column='PersonalPoblacion', max_length=100, null=True)),
                ('personal_estado', models.CharField(blank=True, db_column='PersonalEstado', max_length=30, null=True)),
                ('personal_pais', models.CharField(blank=True, db_column='PersonalPais', max_length=30, null=True)),
                ('personal_zona', models.CharField(blank=True, db_column='PersonalZona', max_length=30, null=True)),
                ('personal_codigo_postal', models.CharField(blank=True, db_column='PersonalCodigoPostal', max_length=15, null=True)),
                ('personal_telefonos', models.CharField(blank=True, db_column='PersonalTelefonos', max_length=100, null=True)),
                ('personal_telefonos_lada', models.CharField(blank=True, db_column='PersonalTelefonosLada', max_length=6, null=True)),
                ('personal_telefono_movil', models.CharField(blank=True, db_column='PersonalTelefonoMovil', max_length=30, null=True)),
                ('personal_sms', models.BooleanField(blank=True, db_column='PersonalSMS', null=True)),
                ('fecha_nacimiento', models.DateTimeField(blank=True, db_column='FechaNacimiento', null=True)),
                ('sexo', models.CharField(blank=True, db_column='Sexo', max_length=20, null=True)),
                ('fecha1', models.DateTimeField(blank=True, db_column='Fecha1', null=True)),
                ('fecha2', models.DateTimeField(blank=True, db_column='Fecha2', null=True)),
                ('fecha3', models.DateTimeField(blank=True, db_column='Fecha3', null=True)),
                ('fecha4', models.DateTimeField(blank=True, db_column='Fecha4', null=True)),
                ('fecha5', models.DateTimeField(blank=True, db_column='Fecha5', null=True)),
                ('usuario', models.CharField(blank=True, db_column='Usuario', max_length=10, null=True)),
                ('fiscal_regimen', models.CharField(blank=True, db_column='FiscalRegimen', max_length=30, null=True)),
                ('fiscal_zona', models.CharField(blank=True, db_column='FiscalZona', max_length=30, null=True)),
                ('email', models.CharField(blank=True, db_column='eMail', max_length=50, null=True)),
                ('contactar', models.CharField(blank=True, db_column='Contactar', max_length=30, null=True)),
            ],
            options={
                'db_table': 'Cte',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Empresa',
            fields=[
                ('empresa', models.CharField(db_column='Empresa', max_length=5, primary_key=True, serialize=False)),
                ('nombre', models.CharField(blank=True, db_column='Nombre', max_length=100, null=True)),
                ('grupo', models.CharField(blank=True, db_column='Grupo', max_length=100, null=True)),
                ('direccion', models.CharField(blank=True, db_column='Direccion', max_length=100, null=True)),
                ('direccion_numero', models.CharField(blank=True, db_column='DireccionNumero', max_length=20, null=True)),
                ('direccion_numero_int', models.CharField(blank=True, db_column='DireccionNumeroInt', max_length=20, null=True)),
                ('colonia', models.CharField(blank=True, db_column='Colonia', max_length=100, null=True)),
                ('poblacion', models.CharField(blank=True, db_column='Poblacion', max_length=30, null=True)),
                ('estado', models.CharField(blank=True, db_column='Estado', max_length=30, null=True)),
                ('pais', models.CharField(blank=True, db_column='Pais', max_length=30, null=True)),
                ('codigo_postal', models.CharField(blank=True, db_column='CodigoPostal', max_length=15, null=True)),
                ('telefonos', models.CharField(blank=True, db_column='Telefonos', max_length=100, null=True)),
                ('rfc', models.CharField(blank=True, db_column='RFC', max_length=20, null=True)),
                ('tipo', models.CharField(blank=True, db_column='Tipo', max_length=20, null=True)),
            ],
            options={
                'db_table': 'Empresa',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Sucursal',
            fields=[
                ('sucursal', models.IntegerField(db_column='Sucursal', primary_key=True, serialize=False)),
                ('nombre', models.CharField(blank=True, db_column='Nombre', max_length=100, null=True)),
                ('prefijo', models.CharField(blank=True, db_column='Prefijo', max_length=5, null=True)),
                ('estatus', models.CharField(db_column='Estatus', max_length=15)),
                ('rfc', models.CharField(blank=True, db_column='RFC', max_length=20, null=True)),
                ('alta', models.DateTimeField(blank=True, db_column='Alta', null=True)),
                ('almacen_principal', models.CharField(blank=True, db_column='AlmacenPrincipal', max_length=10, null=True)),
                ('cliente', models.CharField(blank=True, db_column='Cliente', max_length=10, null=True)),
                ('categoria', models.CharField(blank=True, db_column='Categoria', max_length=50, null=True)),
                ('ip', models.CharField(blank=True, db_column='IP', max_length=20, null=True)),
                ('fiscal_regimen', models.CharField(blank=True, db_column='FiscalRegimen', max_length=30, null=True)),
                ('version', models.CharField(blank=True, db_column='Version', max_length=25, null=True)),
            ],
            options={
                'db_table': 'Sucursal',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='TipoOrdenOperacion',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('interfaz', models.CharField(db_column='Interfaz', max_length=100)),
                ('operacion_planta', models.CharField(db_column='OrdenOperacionPlanta', max_length=50)),
                ('operacion_intelisis', models.CharField(db_column='OrdenOperacionIntelisis', max_length=50)),
            ],
            options={
                'db_table': 'CA_MapeoTipoOrdenOperacion',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Venta',
            fields=[
                ('id', models.AutoField(db_column='ID', primary_key=True, serialize=False)),
                ('empresa', models.CharField(db_column='Empresa', max_length=5)),
                ('mov', models.CharField(db_column='Mov', max_length=20)),
                ('mov_id', models.CharField(blank=True, db_column='MovID', max_length=20, null=True)),
                ('fecha_emision', models.DateTimeField(blank=True, db_column='FechaEmision', null=True)),
                ('ultimo_cambio', models.DateTimeField(blank=True, db_column='UltimoCambio', null=True)),
                ('concepto', models.CharField(blank=True, db_column='Concepto', max_length=50, null=True)),
                ('uen', models.IntegerField(blank=True, db_column='UEN', null=True)),
                ('moneda', models.CharField(db_column='Moneda', max_length=10)),
                ('tipo_cambio', models.FloatField(blank=True, db_column='TipoCambio', null=True)),
                ('usuario', models.CharField(blank=True, db_column='Usuario', max_length=10, null=True)),
                ('referencia', models.CharField(blank=True, db_column='Referencia', max_length=50, null=True)),
                ('observaciones', models.CharField(blank=True, db_column='Observaciones', max_length=100, null=True)),
                ('estatus', models.CharField(blank=True, db_column='Estatus', max_length=15, null=True)),
                ('situacion', models.CharField(blank=True, db_column='Situacion', max_length=50, null=True)),
                ('situacion_fecha', models.DateTimeField(blank=True, db_column='SituacionFecha', null=True)),
                ('situacion_usuario', models.CharField(blank=True, db_column='SituacionUsuario', max_length=10, null=True)),
                ('situacion_nota', models.CharField(blank=True, db_column='SituacionNota', max_length=100, null=True)),
                ('cliente', models.CharField(db_column='Cliente', max_length=10)),
                ('almacen', models.CharField(db_column='Almacen', max_length=10)),
                ('agente', models.CharField(blank=True, db_column='Agente', max_length=10, null=True)),
                ('agente_servicio', models.CharField(blank=True, db_column='AgenteServicio', max_length=10, null=True)),
                ('fecha_requerida', models.DateTimeField(blank=True, db_column='FechaRequerida', null=True)),
                ('hora_requerida', models.CharField(blank=True, db_column='HoraRequerida', max_length=5, null=True)),
                ('condicion', models.CharField(blank=True, db_column='Condicion', max_length=50, null=True)),
                ('servicio_tipo', models.CharField(blank=True, db_column='ServicioTipo', max_length=50, null=True)),
                ('servicio_articulo', models.CharField(blank=True, db_column='ServicioArticulo', max_length=20, null=True)),
                ('servicio_serie', models.CharField(blank=True, db_column='ServicioSerie', max_length=50, null=True)),
                ('servicio_contrato', models.CharField(blank=True, db_column='ServicioContrato', max_length=20, null=True)),
                ('servicio_contrato_id', models.CharField(blank=True, db_column='ServicioContratoID', max_length=20, null=True)),
                ('servicio_contrato_tipo', models.CharField(blank=True, db_column='ServicioContratoTipo', max_length=50, null=True)),
                ('servicio_descripcion', models.CharField(blank=True, db_column='ServicioDescripcion', max_length=100, null=True)),
                ('servicio_fecha', models.DateTimeField(blank=True, db_column='ServicioFecha', null=True)),
                ('servicio_flotilla', models.BooleanField(blank=True, db_column='ServicioFlotilla', null=True)),
                ('servicio_rampa', models.BooleanField(blank=True, db_column='ServicioRampa', null=True)),
                ('servicio_identificador', models.CharField(blank=True, db_column='ServicioIdentificador', max_length=20, null=True)),
                ('servicio_placas', models.CharField(blank=True, db_column='ServicioPlacas', max_length=20, null=True)),
                ('servicio_kms', models.IntegerField(blank=True, db_column='ServicioKms', null=True)),
                ('servicio_tipo_orden', models.CharField(blank=True, db_column='ServicioTipoOrden', max_length=20, null=True)),
                ('servicio_tipo_operacion', models.CharField(blank=True, db_column='ServicioTipoOperacion', max_length=50, null=True)),
                ('servicio_siniestro', models.CharField(blank=True, db_column='ServicioSiniestro', max_length=20, null=True)),
                ('servicio_deducible_importe', models.DecimalField(blank=True, db_column='ServicioDeducibleImporte', decimal_places=4, max_digits=19, null=True)),
                ('servicio_numero', models.FloatField(blank=True, db_column='ServicioNumero', null=True)),
                ('servicio_numero_economico', models.CharField(blank=True, db_column='ServicioNumeroEconomico', max_length=20, null=True)),
                ('servicio_aseguradora', models.CharField(blank=True, db_column='ServicioAseguradora', max_length=10, null=True)),
                ('servicio_puntual', models.BooleanField(blank=True, db_column='ServicioPuntual', null=True)),
                ('servicio_poliza', models.CharField(blank=True, db_column='ServicioPoliza', max_length=20, null=True)),
                ('origen', models.CharField(blank=True, db_column='Origen', max_length=20, null=True)),
                ('origen_id', models.CharField(blank=True, db_column='OrigenID', max_length=20, null=True)),
                ('servicio_modelo', models.CharField(blank=True, db_column='ServicioModelo', max_length=4, null=True)),
                ('sucursal', models.IntegerField(db_column='Sucursal')),
                ('sucursal_destino', models.IntegerField(blank=True, db_column='SucursalDestino', null=True)),
                ('comentarios', models.TextField(blank=True, db_column='Comentarios', null=True)),
                ('fecha_entrega', models.DateTimeField(blank=True, db_column='FechaEntrega', null=True)),
                ('hora_recepcion', models.CharField(blank=True, db_column='HoraRecepcion', max_length=5, null=True)),
            ],
            options={
                'db_table': 'Venta',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='VentaD',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('id_venta', models.IntegerField(db_column='ID')),
                ('renglon', models.FloatField(db_column='Renglon')),
                ('renglon_sub', models.IntegerField(db_column='RenglonSub')),
                ('renglon_id', models.IntegerField(blank=True, db_column='RenglonID', null=True)),
                ('renglon_tipo', models.CharField(blank=True, db_column='RenglonTipo', max_length=1, null=True)),
                ('cantidad', models.FloatField(blank=True, db_column='Cantidad', null=True)),
                ('almacen', models.CharField(blank=True, db_column='Almacen', max_length=10, null=True)),
                ('articulo', models.CharField(db_column='Articulo', max_length=20)),
                ('precio', models.FloatField(blank=True, db_column='Precio', null=True)),
                ('precio_sugerido', models.FloatField(blank=True, db_column='PrecioSugerido', null=True)),
                ('impuesto1', models.FloatField(blank=True, db_column='Impuesto1', null=True)),
                ('impuesto2', models.FloatField(blank=True, db_column='Impuesto2', null=True)),
                ('impuesto3', models.FloatField(blank=True, db_column='Impuesto3', null=True)),
                ('descripcion_extra', models.CharField(blank=True, db_column='DescripcionExtra', max_length=100, null=True)),
                ('costo', models.DecimalField(blank=True, db_column='Costo', decimal_places=4, max_digits=19, null=True)),
                ('aplica', models.CharField(blank=True, db_column='Aplica', max_length=20, null=True)),
                ('aplica_id', models.CharField(blank=True, db_column='AplicaID', max_length=20, null=True)),
                ('unidad', models.CharField(blank=True, db_column='Unidad', max_length=50, null=True)),
                ('fecha_requerida', models.DateTimeField(blank=True, db_column='FechaRequerida', null=True)),
                ('hora_requerida', models.CharField(blank=True, db_column='HoraRequerida', max_length=5, null=True)),
                ('agente', models.CharField(blank=True, db_column='Agente', max_length=10, null=True)),
                ('departamento', models.IntegerField(blank=True, db_column='Departamento', null=True)),
                ('sucursal', models.IntegerField(db_column='Sucursal')),
                ('sucursal_origen', models.IntegerField(blank=True, db_column='SucursalOrigen', null=True)),
                ('uen', models.IntegerField(blank=True, db_column='UEN', null=True)),
                ('precio_lista', models.DecimalField(blank=True, db_column='PrecioLista', decimal_places=4, max_digits=19, null=True)),
                ('tipo_impuesto1', models.CharField(blank=True, db_column='TipoImpuesto1', max_length=10, null=True)),
                ('tipo_impuesto2', models.CharField(blank=True, db_column='TipoImpuesto2', max_length=10, null=True)),
                ('tipo_impuesto3', models.CharField(blank=True, db_column='TipoImpuesto3', max_length=10, null=True)),
                ('retencion1', models.FloatField(blank=True, db_column='Retencion1', null=True)),
                ('retencion2', models.FloatField(blank=True, db_column='Retencion2', null=True)),
                ('retencion3', models.FloatField(blank=True, db_column='Retencion3', null=True)),
                ('tipo_retencion1', models.CharField(blank=True, db_column='TipoRetencion1', max_length=10, null=True)),
                ('tipo_retencion2', models.CharField(blank=True, db_column='TipoRetencion2', max_length=10, null=True)),
                ('tipo_retencion3', models.CharField(blank=True, db_column='TipoRetencion3', max_length=10, null=True)),
                ('comentarios', models.CharField(blank=True, db_column='Comentarios', max_length=250, null=True)),
                ('servicio_tipo_orden', models.CharField(blank=True, db_column='ServicioTipoOrden', max_length=20, null=True)),
                ('articulo_actual', models.CharField(blank=True, db_column='ArticuloActual', max_length=20, null=True)),
                ('ut', models.FloatField(blank=True, db_column='UT', null=True)),
                ('cc_tiempo_tab', models.FloatField(blank=True, db_column='CCTiempoTab', null=True)),
            ],
            options={
                'db_table': 'VentaD',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='Vin',
            fields=[
                ('vin', models.CharField(db_column='VIN', max_length=20, primary_key=True, serialize=False)),
                ('articulo', models.CharField(blank=True, db_column='Articulo', max_length=20, null=True)),
                ('km', models.IntegerField(blank=True, db_column='Km', null=True)),
                ('motor', models.CharField(blank=True, db_column='Motor', max_length=20, null=True)),
                ('fecha', models.DateTimeField(blank=True, db_column='Fecha', null=True)),
                ('cliente', models.CharField(blank=True, db_column='Cliente', max_length=10, null=True)),
                ('conductor', models.CharField(blank=True, db_column='Conductor', max_length=10, null=True)),
                ('alta', models.DateTimeField(blank=True, db_column='Alta', null=True)),
                ('empresa', models.CharField(blank=True, db_column='Empresa', max_length=5, null=True)),
                ('placas', models.CharField(blank=True, db_column='Placas', max_length=20, null=True)),
                ('garantia_vencimiento', models.DateTimeField(blank=True, db_column='GarantiaVencimiento', null=True)),
                ('registro', models.CharField(blank=True, db_column='Registro', max_length=20, null=True)),
                ('fecha_carta_credito', models.DateTimeField(blank=True, db_column='FechaCartaCredito', null=True)),
                ('fecha_factura', models.DateTimeField(blank=True, db_column='FechaFactura', null=True)),
                ('fecha_ultimo_servicio', models.DateTimeField(blank=True, db_column='FechaUltimoServicio', null=True)),
                ('fecha_siguiente_servicio', models.DateTimeField(blank=True, db_column='FechaSiguienteServicio', null=True)),
                ('costo', models.FloatField(blank=True, db_column='Costo', null=True)),
                ('modelo', models.CharField(blank=True, db_column='Modelo', max_length=4, null=True)),
                ('tipo_compra', models.CharField(blank=True, db_column='TipoCompra', max_length=1, null=True)),
                ('folio_factura_compra', models.CharField(blank=True, db_column='FolioFacturaCompra', max_length=15, null=True)),
                ('fecha_factura_compra', models.DateTimeField(blank=True, db_column='FechaFacturaCompra', null=True)),
                ('descripcion1', models.CharField(blank=True, db_column='Descripcion1', max_length=38, null=True)),
                ('descripcion2', models.CharField(blank=True, db_column='Descripcion2', max_length=38, null=True)),
                ('color_exterior', models.CharField(blank=True, db_column='ColorExterior', max_length=10, null=True)),
                ('color_exterior_descripcion', models.CharField(blank=True, db_column='ColorExteriorDescripcion', max_length=50, null=True)),
                ('color_interior', models.CharField(blank=True, db_column='ColorInterior', max_length=10, null=True)),
                ('color_interior_descripcion', models.CharField(blank=True, db_column='ColorInteriorDescripcion', max_length=50, null=True)),
                ('fecha_pago', models.DateTimeField(blank=True, db_column='FechaPago', null=True)),
                ('venta_id', models.IntegerField(blank=True, db_column='VentaID', null=True)),
                ('compra_id', models.IntegerField(blank=True, db_column='CompraID', null=True)),
                ('estatus', models.CharField(blank=True, db_column='Estatus', max_length=15, null=True)),
                ('situacion', models.CharField(blank=True, db_column='Situacion', max_length=50, null=True)),
                ('situacion_fecha', models.DateTimeField(blank=True, db_column='SituacionFecha', null=True)),
                ('agente', models.CharField(blank=True, db_column='Agente', max_length=10, null=True)),
                ('tipo_venta', models.CharField(blank=True, db_column='TipoVenta', max_length=1, null=True)),
                ('kilometraje_inicial', models.IntegerField(blank=True, db_column='KilometrajeInicial', null=True)),
                ('cilindros', models.IntegerField(blank=True, db_column='Cilindros', null=True)),
                ('puertas', models.CharField(blank=True, db_column='Puertas', max_length=30, null=True)),
                ('pasajeros', models.IntegerField(blank=True, db_column='Pasajeros', null=True)),
                ('capacidad_carga', models.IntegerField(blank=True, db_column='CapacidadCarga', null=True)),
                ('combustible', models.CharField(blank=True, db_column='Combustible', max_length=20, null=True)),
                ('primera_llamada', models.DateTimeField(blank=True, db_column='PrimeraLlamada', null=True)),
                ('comentarios_primera_llamada', models.CharField(blank=True, db_column='ComentariosPrimeraLLamada', max_length=1000, null=True)),
                ('segunda_llamada', models.DateTimeField(blank=True, db_column='SegundaLlamada', null=True)),
                ('comentarios_segunda_llamada', models.CharField(blank=True, db_column='ComentariosSegundaLLamada', max_length=1000, null=True)),
                ('tercera_llamada', models.DateTimeField(blank=True, db_column='TerceraLlamada', null=True)),
                ('comentarios_tercera_llamada', models.CharField(blank=True, db_column='ComentariosTerceraLLamada', max_length=1000, null=True)),
                ('agencia', models.CharField(blank=True, db_column='Agencia', max_length=50, null=True)),
                ('estado', models.CharField(blank=True, db_column='Estado', max_length=50, null=True)),
                ('ciudad', models.CharField(blank=True, db_column='Ciudad', max_length=50, null=True)),
            ],
            options={
                'db_table': 'VIN',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ApiLogs',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('iduser', models.IntegerField()),
                ('tipo', models.CharField(max_length=120)),
                ('header', models.TextField()),
                ('request', models.TextField()),
                ('response', models.TextField()),
                ('url', models.TextField()),
                ('interfaz', models.CharField(max_length=120)),
                ('fecharegistro', models.DateTimeField(auto_now=True)),
            ],
            options={
                'db_table': 'api_logs',
            },
        ),
        migrations.CreateModel(
            name='SepaBranch',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('clave', models.CharField(blank=True, max_length=128)),
                ('nombre', models.CharField(blank=True, max_length=100)),
                ('telefono', models.CharField(blank=True, max_length=40)),
                ('correo', models.CharField(blank=True, max_length=240)),
                ('fecha_creacion', models.DateTimeField(auto_now_add=True)),
                ('fecha_actualizacion', models.DateTimeField(auto_now_add=True)),
                ('conf_ip_ext', models.CharField(blank=True, max_length=40)),
                ('conf_ip_int', models.CharField(blank=True, max_length=40, null=True)),
                ('conf_user', models.CharField(blank=True, max_length=40)),
                ('conf_pass', models.CharField(blank=True, max_length=40)),
                ('conf_db', models.CharField(blank=True, max_length=80)),
                ('conf_port', models.CharField(blank=True, max_length=80, null=True)),
                ('id_intelisis', models.IntegerField(blank=True)),
                ('empresa_intelisis', models.CharField(blank=True, max_length=80)),
                ('foto', models.ImageField(null=True, upload_to='branch')),
                ('foto_alt', models.ImageField(null=True, upload_to='branch_alt')),
                ('eliminado', models.BooleanField(default=False)),
                ('direccion', models.CharField(blank=True, max_length=500, null=True)),
                ('latitud', models.CharField(blank=True, max_length=50, null=True)),
                ('longitud', models.CharField(blank=True, max_length=50, null=True)),
                ('gwmbac', models.CharField(blank=True, max_length=50, null=True)),
                ('id_district', models.IntegerField(blank=True, null=True)),
                ('fotos_recepcion', models.CharField(blank=True, max_length=500, null=True)),
                ('db_recepcion', models.CharField(blank=True, max_length=50, null=True)),
                ('ciudad', models.CharField(blank=True, max_length=40, null=True)),
                ('id_agencia_crm', models.IntegerField(blank=True, null=True)),
                ('version', models.IntegerField(blank=True, null=True)),
            ],
            options={
                'db_table': 'api_sepa_branch',
            },
        ),
        migrations.CreateModel(
            name='SepaCompanies',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('clave', models.CharField(blank=True, max_length=128)),
                ('razonsocial', models.CharField(max_length=100)),
                ('rfc', models.CharField(max_length=40)),
                ('fecha_creacion', models.DateTimeField(auto_now_add=True)),
                ('fecha_actualizacion', models.DateTimeField(auto_now_add=True)),
                ('id_intelisis', models.IntegerField(blank=True)),
                ('foto', models.ImageField(upload_to='companies')),
                ('foto_alt', models.ImageField(upload_to='companies_alt')),
                ('eliminado', models.BooleanField(default=False)),
            ],
            options={
                'db_table': 'api_sepa_companies',
            },
        ),
        migrations.CreateModel(
            name='SepaCurrentVersion',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('nombre', models.CharField(blank=True, max_length=120)),
                ('features', models.CharField(blank=True, max_length=120)),
                ('fixes', models.TextField(blank=True, max_length=2400)),
                ('fecha_creacion', models.DateTimeField(auto_now_add=True)),
                ('fecha_actualizacion', models.DateTimeField(auto_now_add=True)),
                ('eliminado', models.BooleanField(default=False)),
            ],
            options={
                'db_table': 'api_sepa_current_version',
            },
        ),
        migrations.CreateModel(
            name='UserAPI',
            fields=[
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('id', models.AutoField(primary_key=True, serialize=False)),
                ('code_verification', models.CharField(blank=True, max_length=20)),
                ('usuario', models.CharField(blank=True, max_length=254, unique=True)),
                ('password', models.CharField(blank=True, max_length=100)),
                ('nombres', models.CharField(blank=True, max_length=180)),
                ('ap_paterno', models.CharField(blank=True, max_length=100)),
                ('ap_materno', models.CharField(blank=True, max_length=100)),
                ('telefono', models.CharField(blank=True, max_length=40)),
                ('correo', models.CharField(blank=True, max_length=240)),
                ('fecha_creacion', models.DateTimeField(auto_now_add=True)),
                ('fecha_actualizacion', models.DateTimeField(auto_now_add=True)),
                ('id_intelisis', models.CharField(blank=True, max_length=100, null=True)),
                ('id_ckt', models.CharField(blank=True, max_length=100, null=True)),
                ('token_device', models.CharField(blank=True, max_length=100, null=True)),
                ('foto', models.ImageField(upload_to='users')),
                ('foto_alt', models.ImageField(upload_to='users_alt')),
                ('eliminado', models.BooleanField(default=False)),
                ('api_sctoken', models.CharField(blank=True, max_length=150)),
                ('api_scLastLoggedIn', models.DateTimeField(auto_now_add=True)),
                ('api_scInserted', models.BooleanField(default=False)),
                ('is_active', models.BooleanField(default=True)),
                ('is_staff', models.BooleanField(default=False)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
                ('id_branch', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='users', to='isapilib.sepabranch')),
                ('id_current_version', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='users', to='isapilib.sepacurrentversion')),
            ],
            options={
                'db_table': 'api_user',
            },
        ),
        migrations.CreateModel(
            name='SepaBranchUsers',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('fecharegistro', models.DateTimeField(auto_now_add=True)),
                ('fechaactualiza', models.DateTimeField(auto_now=True)),
                ('eliminado', models.BooleanField(default=False)),
                ('idbranch', models.ForeignKey(db_column='idbranch', on_delete=django.db.models.deletion.CASCADE, related_name='permissions', to='isapilib.sepabranch')),
                ('iduser', models.ForeignKey(db_column='iduser', on_delete=django.db.models.deletion.CASCADE, related_name='permissions', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'api_branch_users',
            },
        ),
        migrations.AddField(
            model_name='sepabranch',
            name='id_company',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='branches', to='isapilib.sepacompanies'),
        ),
    ]



from functools import reduce

from django.conf import settings
from django.db import models
from rest_framework.fields import empty, Field

from isapilib.core.exceptions import AbortCreationException
from isapilib.external.utilities import get_param_sucursal, get_param_empresa


class JsonField(Field):
    def __init__(self, json_path=None, return_type=None, prefix=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.json_path = json_path
        self.return_type = return_type or str
        self.prefix = prefix

    def _get_json_value(self, dictionary):
        real_json_path = self.json_path or self.field_name

        try:
            return reduce(lambda d, key: d[key], real_json_path.split("."), dictionary)
        except (KeyError, TypeError):
            return empty

    def get_value(self, dictionary):
        value = self._get_json_value(dictionary)
        if self.prefix:
            value = f'{self.prefix}{value}'
        return value

    def to_internal_value(self, data):
        return self.return_type(data)

    def to_representation(self, value):
        return self.return_type(value)


class SerializerSetMethodField(Field):
    def __init__(self, method_name_get=None, method_name_set=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.method_name_get = method_name_get
        self.method_name_set = method_name_set
        self._value = empty

    def bind(self, field_name, parent):
        super().bind(field_name, parent)
        if self.method_name_get is None: self.method_name_get = f'get_{field_name}'
        if self.method_name_set is None: self.method_name_set = f'set_{field_name}'

    def get_value(self, dictionary):
        if self._value is not empty: return self._value
        method = getattr(self.parent, self.method_name_set)
        self._value = method(dictionary)
        return self._value

    def to_internal_value(self, data):
        return data

    def to_representation(self, value):
        if isinstance(value, (str, int, float)):
            return value
        if issubclass(type(value), models.Model):
            return value.pk
        return None


class GetOrField(JsonField):
    def __init__(self, model, model_field=None, related=True, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.model = model
        self.model_field = model_field
        self.related = related

    def bind(self, field_name, parent):
        super().bind(field_name, parent)
        if self.model_field is None: self.model_field = field_name

    def to_internal_value(self, data):
        instance = self.model.objects.filter(**{self.model_field: data}).first()
        if instance is None: return empty
        return instance if self.related else instance.pk

    def to_representation(self, value):
        return getattr(value, 'pk', value)


class GetOrDefaultField(GetOrField):
    def __init__(self, sucursal_param=None, empresa_param=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.sucursal_param = sucursal_param
        self.empresa_param = empresa_param

    def get_value(self, dictionary):
        value = super().get_value(dictionary)
        if value is not empty: return value

        pk = empty
        if self.sucursal_param:
            instance = self.parent.get_instance()
            pk = get_param_sucursal(instance.sucursal, self.sucursal_param)

        if self.empresa_param:
            interfaz = getattr(settings, 'INTERFAZ_NAME', '')
            pk = get_param_empresa(interfaz, self.empresa_param)

        return pk


class GetOrCreateField(GetOrField):
    def __init__(self, method_name=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.method_name = method_name

    def bind(self, field_name, parent):
        super().bind(field_name, parent)
        if self.method_name is None: self.method_name = f'create_{field_name}'

    def to_internal_value(self, data):
        value = super().to_internal_value(data)
        if value is empty:
            try:
                method = getattr(self.parent, self.method_name)
                value = method(self.parent.initial_data)
                if not isinstance(value, self.model):
                    raise ValueError(f'The function {self.method_name} did not return an instance of the model')
                value.save(triggers=False)
            except AbortCreationException:
                return None
        return value



from django.conf import settings
from django.contrib.auth.base_user import BaseUserManager
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver


class SepaCurrentVersion(models.Model):
    id = models.AutoField(primary_key=True)
    nombre = models.CharField(max_length=120, blank=True)
    features = models.CharField(max_length=120, blank=True)
    fixes = models.TextField(max_length=2400, blank=True)
    fecha_creacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    fecha_actualizacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    eliminado = models.BooleanField(default=False)

    def __str__(self):
        return self.nombre

    class Meta:
        db_table = 'api_sepa_current_version'


class SepaCompanies(models.Model):
    id = models.AutoField(primary_key=True)
    clave = models.CharField(max_length=128, blank=True)
    razonsocial = models.CharField(max_length=100)
    rfc = models.CharField(max_length=40)
    fecha_creacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    fecha_actualizacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    id_intelisis = models.IntegerField(blank=True)
    foto = models.ImageField(upload_to='companies')
    foto_alt = models.ImageField(upload_to='companies_alt')
    eliminado = models.BooleanField(default=False)

    def __str__(self):
        return self.clave

    class Meta:
        db_table = 'api_sepa_companies'


class SepaBranch(models.Model):
    id = models.AutoField(primary_key=True)
    id_company = models.ForeignKey(SepaCompanies, on_delete=models.CASCADE, related_name='branches')
    clave = models.CharField(max_length=128, blank=True)
    nombre = models.CharField(max_length=100, blank=True)
    telefono = models.CharField(max_length=40, blank=True)
    correo = models.CharField(max_length=240, blank=True)
    fecha_creacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    fecha_actualizacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    conf_ip_ext = models.CharField(max_length=40, blank=True)
    conf_ip_int = models.CharField(max_length=40, blank=True, null=True)
    conf_user = models.CharField(max_length=40, blank=True)
    conf_pass = models.CharField(max_length=40, blank=True)
    conf_db = models.CharField(max_length=80, blank=True)
    conf_port = models.CharField(max_length=80, blank=True, null=True)
    id_intelisis = models.IntegerField(blank=True)
    empresa_intelisis = models.CharField(max_length=80, blank=True)
    foto = models.ImageField(upload_to='branch', null=True)
    foto_alt = models.ImageField(upload_to='branch_alt', null=True)
    eliminado = models.BooleanField(default=False)
    direccion = models.CharField(max_length=500, blank=True, null=True)
    latitud = models.CharField(max_length=50, blank=True, null=True)
    longitud = models.CharField(max_length=50, blank=True, null=True)
    gwmbac = models.CharField(max_length=50, blank=True, null=True)
    id_district = models.IntegerField(blank=True, null=True)
    fotos_recepcion = models.CharField(max_length=500, blank=True, null=True)
    db_recepcion = models.CharField(max_length=50, blank=True, null=True)
    ciudad = models.CharField(max_length=40, blank=True, null=True)
    id_agencia_crm = models.IntegerField(blank=True, null=True)
    version = models.IntegerField(blank=True, null=True)

    def __str__(self):
        return self.clave

    class Meta:
        db_table = 'api_sepa_branch'


class UserProfileManager(BaseUserManager):
    def create_user(self, id_branch, id_current_version, code_verification, correo, telefono, usuario, password=None):
        if not correo:
            raise ValueError('El correo es un campo obligatorio')

        correo = self.normalize_email(correo)
        user = self.model(id_branch_id=id_branch, id_current_version_id=id_current_version,
                          code_verification=code_verification, correo=correo, telefono=telefono, usuario=usuario)

        user.set_password(password)
        user.save(using=self._db)

        return user

    def create_superuser(self, id_branch, id_current_version, code_verification, correo, telefono, usuario, password):
        user = self.create_user(id_branch, id_current_version, code_verification, correo, telefono, usuario, password)

        user.is_superuser = True
        user.is_staff = True
        user.save(using=self._db)

        return user


class UserAPI(AbstractBaseUser, PermissionsMixin):
    id = models.AutoField(primary_key=True)
    id_branch = models.ForeignKey(SepaBranch, to_field='id', on_delete=models.CASCADE, related_name='users')
    id_current_version = models.ForeignKey(SepaCurrentVersion, on_delete=models.CASCADE, related_name='users')
    code_verification = models.CharField(max_length=20, blank=True)
    usuario = models.CharField(max_length=254, blank=True, unique=True)
    password = models.CharField(max_length=100, blank=True)
    nombres = models.CharField(max_length=180, blank=True)
    ap_paterno = models.CharField(max_length=100, blank=True)
    ap_materno = models.CharField(max_length=100, blank=True)
    telefono = models.CharField(max_length=40, blank=True)
    correo = models.CharField(max_length=240, blank=True)
    fecha_creacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    fecha_actualizacion = models.DateTimeField(auto_now=False, auto_now_add=True)
    id_intelisis = models.CharField(max_length=100, blank=True, null=True)
    id_ckt = models.CharField(max_length=100, blank=True, null=True)
    token_device = models.CharField(max_length=100, blank=True, null=True)
    foto = models.ImageField(upload_to='users')
    foto_alt = models.ImageField(upload_to='users_alt')
    eliminado = models.BooleanField(default=False)
    api_sctoken = models.CharField(max_length=150, blank=True)
    api_scLastLoggedIn = models.DateTimeField(auto_now=False, auto_now_add=True)
    api_scInserted = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    is_staff = models.BooleanField(default=False)
    is_admin = models.BooleanField(default=False)

    objects = UserProfileManager()

    USERNAME_FIELD = 'usuario'
    REQUIRED_FIELDS = ['id_branch', 'id_current_version', 'code_verification', 'correo', 'telefono', 'password']

    def get_full_name(self):
        return self.nombres

    def get_short_name(self):
        return self.nombres

    def __str__(self):
        return self.usuario

    class Meta:
        db_table = 'api_user'


class SepaBranchUsers(models.Model):
    iduser = models.ForeignKey(UserAPI, on_delete=models.CASCADE, related_name='permissions', db_column='iduser')
    idbranch = models.ForeignKey(SepaBranch, on_delete=models.CASCADE, related_name='permissions', db_column='idbranch')
    fecharegistro = models.DateTimeField(auto_now_add=True)
    fechaactualiza = models.DateTimeField(auto_now=True)
    eliminado = models.BooleanField(default=False)

    class Meta:
        db_table = 'api_branch_users'


class ApiLogs(models.Model):
    iduser = models.IntegerField()
    tipo = models.CharField(max_length=120)
    header = models.TextField()
    request = models.TextField()
    response = models.TextField()
    status = models.IntegerField(null=True)
    url = models.TextField()
    interfaz = models.CharField(max_length=120)
    fecharegistro = models.DateTimeField(auto_now=True)
    response_time = models.IntegerField(default=0)

    class Meta:
        db_table = 'api_logs'


@receiver(post_save, sender=ApiLogs)
def delete_old_logs(sender, instance, **kwargs):
    interfaz = getattr(settings, 'INTERFAZ_NAME', '')
    max_number_logs = getattr(settings, 'MAX_NUMBER_LOGS', 500)
    if max_number_logs < ApiLogs.objects.count():
        logs_to_delete = ApiLogs.objects.filter(interfaz=interfaz).order_by('-fecharegistro')[max_number_logs:]
        list(map(lambda i: i.delete(), logs_to_delete))



from rest_framework import serializers

from isapilib import validators
from isapilib.core.utilities import get_sucursal_from_request
from isapilib.external.utilities import get_sucursal, get_uen, get_almacen
from isapilib.fields import SerializerSetMethodField
from isapilib.mixin.instance import InstanceMixIn
from isapilib.models import Venta, Empresa, Vin


class VentaSerializer(InstanceMixIn, serializers.ModelSerializer):
    sucursal = SerializerSetMethodField()
    empresa = SerializerSetMethodField()
    almacen = SerializerSetMethodField()
    uen = SerializerSetMethodField()

    class Meta:
        model = Venta
        fields = '__all__'
        validators = [
            validators.DateAfterTodayValidator(),
            validators.NoDuplicateAppointmentsValidator(),
        ]

    def set_sucursal(self, data):
        instance: Venta = self.get_instance()
        request = self.context.get('request')
        return get_sucursal(mov=instance.mov, sucursal=get_sucursal_from_request(request))

    def set_empresa(self, data):
        return Empresa.objects.all().first()

    def set_almacen(self, data):
        instance: Venta = self.get_instance()
        return get_almacen(mov=instance.mov, sucursal=instance.sucursal.pk)

    def set_uen(self, data):
        instance: Venta = self.get_instance()
        return get_uen(mov=instance.mov, sucursal=instance.sucursal.pk)

    def to_internal_value(self, data):
        internal_value = super().to_internal_value(data)

        if hasattr(self, 'get_vin'):
            vin: Vin = self.get_vin(data)
            if not isinstance(vin, Vin):
                raise ValueError(f'The function get_vin did not return an instance of VIN')
            internal_value['servicio_serie'] = vin.vin
            internal_value['servicio_modelo'] = vin.modelo
            internal_value['servicio_articulo'] = vin.articulo
            internal_value['servicio_placas'] = vin.placas
            internal_value['servicio_kms'] = vin.km
            internal_value['servicio_descripcion'] = vin.color_exterior or 'NEGRO'
            internal_value['servicio_identificador'] = vin.color_exterior or 'NEGRO'

        return internal_value



from django.conf import settings
from django.http import HttpResponse


def index(request):
    url_doc = getattr(settings, 'URL_DOCUMENTATION', '')

    return HttpResponse(f'''
        <style>
            * {{
                margin: 0;
                padding: 0;
                font-family: 'Roboto', sans-serif;
                color: #dfe2e6;
            }}

            body {{
                background-image: radial-gradient(circle, #004b75, #001c26);
                display: flex;
            }}

            div {{
                margin: auto;
                display: flex;
                font-size: 10vw;
                user-select: none;
                text-align: center;
                flex-direction: column;
            }}

            a {{
                opacity: 0.8;
                font-size: 1vw;
                text-decoration: none;
                transition: all 0.3s ease-in-out;
            }}

            a:hover {{
                transform: scale(1.3);
            }}
        </style>

        <div>
            <p>Intelisis</p>
            {
    f'<a href="{url_doc}">Documentation</a>' if url_doc != '' else ''
    }
        </div>
    ''')



def to_bool(s):
    if isinstance(s, bool):
        return s
    if isinstance(s, str):
        s = s.lower()
        if s in ['true', '1', 'yes']:
            return True
        elif s in ['false', '0', 'no']:
            return False
        else:
            return False


def get_sucursal_from_request(request):
    return request.user.id_intelisis or 0



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, List

from django.db import transaction

from isapilib.core.exceptions import TraspasoException
from isapilib.external.utilities import execute_sp

if TYPE_CHECKING:
    from isapilib.models import Venta, VentaD, VentaTraspasarArticulos


class TraspasoItems:
    def __init__(self, order: Venta, estacion: int):
        self.order = order
        self.estacion = estacion
        self.items: List[VentaTraspasarArticulos] = []

        self.clean()

    def clean(self):
        from isapilib.models import VentaTraspasarArticulos
        VentaTraspasarArticulos.objects.filter(venta=self.order.pk, estacion=self.estacion).delete()

    def add_item(self, detail: VentaD):
        from isapilib.models import VentaTraspasarArticulos
        traspaso = VentaTraspasarArticulos.objects.create(
            venta=self.order.pk,
            estacion=self.estacion,
            articulo=detail.articulo,
            cantidad=detail.cantidad,
            precio=detail.precio or 0,
            accion='Agregar',
            descuento_linea=detail.descuento_linea,
            descuento_importe=detail.descuento_importe,
            paquete=detail.paquete,
            aplica_mov=detail.venta.mov,
            aplica_mov_id=detail.venta.mov_id,
            renglon=detail.renglon,
            origen='API',
        )
        self.items.append(traspaso)
        return traspaso

    def execute(self):
        from isapilib.models import VentaTraspasarArticulos
        try:
            with transaction.atomic():
                response = execute_sp('xpVentaTraspasarArticulosProcesar', {
                    'ID': self.order.pk,
                    'Estacion': self.estacion,
                    'Usuario': 'SOPDESA',
                    'FechaTrabajo': datetime.now(),
                    'Conexion': 1,
                    'CA_EnSilencio': 0,
                }, only_output=False, with_transaction=True)[0][0]
                if VentaTraspasarArticulos.objects.filter(venta=self.order.pk, estacion=self.estacion).exists():
                    raise TraspasoException(self.order.pk, self.estacion, response[0])
                return response
        except Exception as e:
            raise e



from __future__ import annotations

import random
from typing import Union, TYPE_CHECKING

from django.db import transaction
from django.db.models import QuerySet

from isapilib.core import exceptions
from isapilib.external.utilities import execute_sp
from isapilib.mov.traspaso import TraspasoItems

if TYPE_CHECKING:
    from isapilib.models import Venta, Art, VentaD


class VentaMixin:
    class ACTIONS:
        AFECTAR = 'AFECTAR'
        CONSECUTIVO = 'CONSECUTIVO'
        GENERAR = 'GENERAR'

    def get_origen(self: Venta) -> Union[Venta, None]:
        from isapilib.models import Venta
        if self.origen is None or self.origen_id is None: return None
        return Venta.objects.filter(mov=self.origen, mov_id=self.origen_id).order_by('-fecha_emision').first()

    def get_destinations(self: Venta) -> Union[Venta, None]:
        from isapilib.models import Venta
        if self.mov is None or self.mov_id is None: return None
        return Venta.objects.filter(origen=self.mov, origen_id=self.mov_id).order_by('-fecha_emision').all()

    def create_parts_order_from_service(self: Venta) -> Venta:
        from isapilib.models import Venta

        try:
            with transaction.atomic():
                if self.mov != 'Servicio': raise ValueError('La instancia de venta no es un servicio')

                if self.estatus != 'PENDIENTE':
                    msg = 'El servicio no esta afectado' if self.estatus == 'SINAFECTAR' else 'El servicio ya esta concluido'
                    raise exceptions.ValidationException(msg)

                estimated_pk = execute_sp('xpCA_xpMovCopiarSoloEncabezado', {
                    'Sucursal': self.sucursal.pk,
                    'Modulo': 'VTAS',
                    'ID': self.id,
                    'Empresa': self.empresa.pk,
                    'Mov': 'Pedido Servicio',
                    'MovID': self.mov_id,
                    'Usuario': self.usuario,
                    'TipoCambio': self.tipo_cambio,
                    'Moneda': self.moneda,
                    'Almacen': self.almacen.pk,
                    'AlmacenDestino': self.almacen.pk,
                })[0]

                estimated = Venta.objects.get(id=estimated_pk)
                estimated.origen = self.mov
                estimated.origen_id = self.mov_id
                estimated.save()
                return estimated
        except Exception as e:
            raise e

    def create_service_from_appointment(self: Venta, action=None) -> Venta:
        from isapilib.models import Venta

        try:
            with transaction.atomic():
                if self.mov != 'Cita Servicio': raise ValueError('La instancia de venta no es una cita servicio')
                order = Venta.objects.filter(origen=self.mov, origen_id=self.mov_id, mov='Servicio').order_by(
                    '-fecha_emision').first()

                if self.estatus == 'CONCLUIDO' or getattr(order, 'estatus', 'SINAFECTAR') in ['PENDIENTE', 'CONCLUIDO']:
                    raise exceptions.ValidationException('La cita ya esta concluida')

                if not self.details.exists():
                    raise exceptions.ValidationException('La cita no tiene detalles')

                if self.estatus == 'SINAFECTAR':
                    self.afectar()

                code, description = self.afectar(accion=self.ACTIONS.GENERAR, generar='Servicio')
                if code != 80030:
                    raise exceptions.CoreException(code, description, self)

                order = Venta.objects.filter(origen=self.mov, origen_id=self.mov_id).order_by('-fecha_emision').first()
                if action: order.afectar(accion=action)
                return order
        except Exception as e:
            raise e

    def traspasar_items(self, details: QuerySet[VentaD]):
        try:
            with transaction.atomic():
                if details.count() == 0: return

                order = self.get_origen()
                estacion = random.randint(10000, 99999)

                traspaso = TraspasoItems(order, estacion)

                for detail in details:
                    traspaso.add_item(detail)

                traspaso.execute()
        except Exception as e:
            raise e

    def accept_parts_order(self: Venta, items: tuple[str, bool]) -> bool:
        from isapilib.models import VentaD

        try:
            with transaction.atomic():
                if self.mov != 'Pedido Servicio': raise ValueError('La instancia de venta no es una pedido servicio')
                items_pk = []
                errors = []

                for art_pk, is_accept in items:
                    try:
                        self.details.get(articulo=art_pk)
                        if is_accept: items_pk.append(art_pk)
                    except VentaD.DoesNotExist:
                        errors.append((art_pk, 'El articulo no existe en el pedido servicio'))

                if 0 < len(errors):
                    raise Exception(errors)

                self.traspasar_items(self.details.filter(articulo__in=items_pk))
        except Exception as e:
            raise e

    def add_item(self: Venta, art: Art, cantidad=1, save=True) -> VentaD:
        from isapilib.models import VentaD
        count = VentaD.objects.filter(venta=self).count() + 1
        detail = VentaD(venta=self, renglon_sub=0, renglon_tipo='N', almacen=self.almacen, uen=self.uen,
                        sucursal=self.sucursal, sucursal_origen=self.sucursal, agente=self.agente,
                        hora_requerida=self.hora_requerida, renglon=2048 * count, renglon_id=count,
                        articulo=art, impuesto1=art.impuesto1, descripcion_extra=art.descripcion1,
                        comentarios=art.descripcion1, precio=art.precio_lista or 0, ut=1, cc_tiempo_tab=1,
                        cantidad=cantidad)
        if save: detail.save(triggers=False)
        return detail

    def afectar(self: Venta, *, accion=None, generar=None, usuario=None, s=0, modulo=None, estacion=None, base=None):
        accion = accion or self.ACTIONS.AFECTAR
        code, description = execute_sp('spAfectar', {
            'Modulo': modulo or 'VTAS',
            'ID': self.id,
            'Accion': accion,
            'Base': base or 'Todo',
            'Usuario': usuario or 'SOPDESA',
            'GenerarMov': generar,
            'EnSilencio': s,
            'Estacion': estacion or 1000,
        })

        self.refresh_from_db()

        if (
                (accion == self.ACTIONS.AFECTAR and self.estatus == 'SINAFECTAR') or
                (accion == self.ACTIONS.CONSECUTIVO and self.mov_id is None)
        ):
            raise exceptions.CoreException(code, description, self)

        return code, description



from django.db import connections

from isapilib.api.models import UserAPI, SepaCurrentVersion, SepaCompanies, SepaBranch, ApiLogs, SepaBranchUsers


def conn_production(fun):
    def wrapper_fun(*args, **kwargs):
        if 'production' not in connections.databases:
            production_database = connections.databases['default']
            production_database['NAME'] = production_database['NAME'].replace('test_', '')
            connections.databases['production'] = production_database
        return fun(*args, **kwargs)

    return wrapper_fun


@conn_production
def get_user(username, password, branch_pk) -> tuple[UserAPI, SepaBranch]:
    current_version = SepaCurrentVersion.objects.create()

    company = SepaCompanies.objects.create(
        id_intelisis=0,
    )

    production_branch = SepaBranch.objects.using('production').get(pk=branch_pk)

    branch = SepaBranch.objects.create(
        id_company=company,
        conf_ip_ext=production_branch.conf_ip_ext,
        conf_ip_int=production_branch.conf_ip_int,
        conf_user=production_branch.conf_user,
        conf_pass=production_branch.conf_pass,
        conf_db=production_branch.conf_db,
        conf_port=production_branch.conf_port,
        id_intelisis=production_branch.id_intelisis,
    )

    user = UserAPI.objects.create_user(
        id_branch=branch.pk,
        id_current_version=current_version.pk,
        code_verification='test',
        correo='test@test.test',
        telefono='1234567890',
        usuario=username,
        password=password,
    )

    SepaBranchUsers.objects.create(iduser=user, idbranch=branch)

    return user, branch


@conn_production
def get_logs(interfaz):
    return ApiLogs.objects.using('production').filter(interfaz=interfaz)



# Generated by Django 5.0.2 on 2024-07-23 18:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('isapilib', '0003_userapi_is_admin_alter_apilogs_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='apilogs',
            name='response_time',
            field=models.IntegerField(default=0),
        ),
    ]



from django.conf import settings
from django.http import JsonResponse


def safe_method(view_func):
    def wrapped_view(*args, **kwargs):
        try:
            return view_func(*args, **kwargs)
        except Exception as e:
            if settings.DEBUG: raise e
            return JsonResponse({
                'type': str(type(e)),
                'message': str(e)
            }, status=500)

    return wrapped_view



import json

from isapilib.api.models import ApiLogs


def get_response_content(response) -> str:
    try:
        return str(response.data)
    except Exception:
        return str(response.content.decode('utf-8'))


def insert_log(request, response, interfaz, tipo, time=0):
    try:
        response_content = get_response_content(response)
        log = ApiLogs()
        log.iduser = request.user.pk
        log.tipo = str(tipo or '')
        log.header = str(json.dumps(dict(request.headers)))
        log.request = str(json.dumps(request.data) if isinstance(request.data, (dict, list)) else request.data or '')
        log.response = str(response_content or '')
        log.status = response.status_code if hasattr(response, 'status_code') else 0
        log.url = request.build_absolute_uri()
        log.interfaz = str(interfaz or '')
        log.response_time = time
        log.save(using='default')
    except Exception as e:
        print(f'Warning: Failed to save logs, ({type(e)}) {e}')



class SepaException(Exception):

    def __init__(self, description, user=None, branch=None):
        self.user = user
        self.branch = branch
        super().__init__(description)


class ValidationException(Exception):
    pass


class CoreException(Exception):
    def __init__(self, code, message, instance=None):
        self.code = code
        self.message = message
        self.instance = instance
        super().__init__(str(self))

    def _get_instance(self):
        try:
            return f' ({self.instance.pk})'
        except Exception:
            return ''

    def __str__(self):
        from isapilib.models import MensajeLista
        message = MensajeLista.objects.filter(mensaje=self.code).first()
        message = f'{message.descripcion}, {self.message}' if message else self.message
        return str(f'{self.code}: {message}' + self._get_instance()).capitalize()


class TraspasoException(Exception):
    def __init__(self, order, estacion, message):
        self.order = order
        self.estacion = estacion
        self.message = message
        super().__init__(str(self))

    def __str__(self):
        return f'{self.message} ({self.order}, {self.estacion})'


class AbortCreationException(Exception):
    pass



from datetime import datetime

from isapilib.models import Venta


class DateAfterTodayValidator:
    requires_context = False

    def __call__(self, value):
        fecha_requerida = value['fecha_requerida']
        current_date = datetime.now(tz=fecha_requerida.tzinfo)
        if fecha_requerida < current_date:
            raise Exception('The date must be after the current date')

        return value


class NoDuplicateAppointmentsValidator:
    requires_context = False

    def __call__(self, value):
        filters = {
            'agente': value['agente'],
            'fecha_requerida__date': value['fecha_requerida'].date(),
            'hora_recepcion': value['hora_recepcion'],
            'estatus': 'CONFIRMAR'
        }
        if Venta.objects.filter(**filters).exists():
            raise Exception('This appointment has already been scheduled')



import threading

from rest_framework.permissions import BasePermission

_thread_local = threading.local()


class IsapilibPermission(BasePermission):
    def has_permission(self, request, view):
        _thread_local.request = request
        return True

    @staticmethod
    def get_current_request():
        return getattr(_thread_local, 'request', None)



import copy
from typing import TypeVar

from django.db import models
from rest_framework.fields import empty, SkipField

SubModel = TypeVar('SubModel', bound=models.Model)


class InstanceMixIn:
    _got_data = {}

    def got(self, name, fun):
        def wrapper(*args, **kwargs):
            data = fun(*args, **kwargs)
            if data is not empty:
                self._got_data[name] = data
            return data

        return wrapper

    def get_fields(self):
        wrapper_fields = copy.deepcopy(self._declared_fields)

        for name, field in wrapper_fields.items():
            get_value_method = self.got(name, getattr(field, 'get_value'))
            setattr(field, 'get_value', get_value_method)
            try:
                self._got_data[name] = field.get_default()
            except SkipField:
                pass

        return wrapper_fields

    def get_instance(self):
        model = self.Meta.model
        instance = model()
        for name, data in self._got_data.items():
            field = self.fields[name]
            value = field.to_internal_value(data)
            setattr(instance, name, value)
        return instance



from isapilib.external.utilities import execute_sp


class CteMixin:
    def new_cte(self):
        self.cliente = execute_sp('spConsecutivo', ['CTE', 0])[0][0]
        self.colonia = 'NA'
        self.rfc = 'XAXX010101000'
        self.direccion = 'NA'
        self.direccion_numero = '1'
        self.tipo = 'Cliente'
        self.estado = 'NA'
        self.pais = 'NA'
        self.estatus = 'ALTA'



from django.apps import apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.db import connections

from isapilib.api.models import SepaBranch
from isapilib.core.exceptions import SepaException


def add_conn(user, branch: SepaBranch):
    try:
        user_model_path = getattr(settings, 'AUTH_USER_MODEL', 'isapilib.UserAPI')
        branch_model_path = getattr(settings, 'BRANCH_MODEL', 'isapilib.SepaBranch')
        permission_model_path = getattr(settings, 'PERMISSION_MODEL', 'isapilib.SepaBranchUsers')

        user_model = apps.get_model(user_model_path, require_ready=False)
        branch_model = apps.get_model(branch_model_path, require_ready=False)
        permission_model = apps.get_model(permission_model_path, require_ready=False)
    except ValueError:
        raise ImproperlyConfigured("AUTH_USER_MODEL must be of the form 'app_label.model_name'")
    except LookupError as e:
        if settings.AUTH_USER_MODEL in str(e):
            raise ImproperlyConfigured(
                f"AUTH_USER_MODEL refers to model '{settings.AUTH_USER_MODEL}' that has not been installed"
            )
        elif settings.BRANCH_MODEL in str(e):
            raise ImproperlyConfigured(
                f"BRANCH_MODEL refers to model '{settings.BRANCH_MODEL}' that has not been installed"
            )
        elif settings.PERMISSION_MODEL in str(e):
            raise ImproperlyConfigured(
                f"PERMISSION_MODEL refers to model '{settings.PERMISSION_MODEL}' that has not been installed"
            )
        else:
            raise e

    try:
        permission_model.objects.get(iduser=user, idbranch=branch)
        conn = f'external-{branch.id}'
        if conn not in connections.databases:
            connections.databases[conn] = create_conn(branch)
        return conn
    except user_model.DoesNotExist:
        raise SepaException('The user does not exist')
    except branch_model.DoesNotExist:
        raise SepaException('The agency does not exist', user)
    except permission_model.DoesNotExist:
        raise SepaException('You do not have permissions on the agency', user, branch)


def get_version(version=6000):
    version = version or 6000

    if 5000 > version >= 4000:
        return '4000'

    return '6000'


def create_conn(_branch):
    return {
        'ENGINE': 'mssql',
        'NAME': _branch.conf_db if _branch.conf_db else '',
        'USER': _branch.conf_user if _branch.conf_user else '',
        'PASSWORD': _branch.conf_pass if _branch.conf_pass else '',
        'HOST': _branch.conf_ip_ext if _branch.conf_ip_ext else '',
        'PORT': _branch.conf_port if _branch.conf_port else '',
        'INTELISIS_VERSION': get_version(_branch.version),
        'TIME_ZONE': None,
        'CONN_HEALTH_CHECKS': None,
        'CONN_MAX_AGE': None,
        'ATOMIC_REQUESTS': None,
        'AUTOCOMMIT': True,
        'OPTIONS': settings.DATABASES['default'].get('OPTIONS')
    }



from isapilib.api.models import SepaBranch
from isapilib.auth.permissions import IsapilibPermission
from isapilib.external.connection import add_conn


class BaseRouter:
    default_database = (
        'SepaCurrentVersion'.lower(),
        'SepaCompanies'.lower(),
        'SepaBranch'.lower(),
        'UserProfileManager'.lower(),
        'UserAPI'.lower(),
        'SepaBranchUsers'.lower(),
        'ApiLogs'.lower(),
        'idtoken',
        'accesstoken',
        'refreshtoken',
        'application',
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_branch(self, user, request) -> SepaBranch:
        return request.user.id_branch

    def external_db(self):
        request = IsapilibPermission.get_current_request()
        branch = self.get_branch(request.user, request)
        return add_conn(request.user, branch)

    def _get_model_name(self, model):
        try:
            return model._meta.model_name
        except Exception as e:
            return 'execution'

    def db_for_read(self, model, **hints):
        if self._get_model_name(model) in self.default_database: return 'default'
        return self.external_db()

    def db_for_write(self, model, **hints):
        if self._get_model_name(model) in self.default_database: return 'default'
        return self.external_db()

    def allow_relation(self, *args, **kwargs):
        return True
